var decodeValueFns = {
  group: decodeList,
  pairwise: decodeList,
  key_mgmt: decodeList,
  auth_alg: decodeList,
  proto: decodeList,
  eap: decodeList,
  config_methods: decodeList,
  non_pref_chan: decodeList,
  scan_freq: decodeList,
  freq_list: decodeList,
  bssid_whitelist: decodeList,
  bssid_blacklist: decodeList,
  p2p_client_list: decodeList,
  driver_param: decodeObj,
  phase1: decodeObj,
  phase2: decodeObj,
  ctrl_interface: function (value) {
    var m = /^"(.*)"$/.exec(value)
    if (/=/.test(value)) return decodeObj(value)
    return value
  },
}

function defaultDecodeValue(value) {
  try {
    return JSON.parse(value)
  } catch(e) {}
  var m = /^"(.*)"$/.exec(value)
  if (m) return m[1]
  return value
}

function decodeList(value) {
  var values = value.split(' ')
  if (values.length == 1) return values[0]
  return values
}

function decodeObj(str) {
  var m = /^"(.*)"$/.exec(str)
  if (m) str = m[1]
  var obj = {}
  str.split(' ').map(decodeProperty).filter(Boolean).forEach(function (prop) {
    setProp(obj, prop.name, prop.value)
  })
  return obj
}

function decodeProperty(line) {
  var m = /^([^=]*)=(.*)$/.exec(line)
  if (!m) return
  var name = m[1]
  var decodeValue = decodeValueFns[name] || defaultDecodeValue
  return {name: name, value: decodeValue(m[2])}
}

function setProp(obj, name, value) {
  if (!(name in obj)) {
    obj[name] = value
  } else if (Array.isArray(obj[name])) {
    obj[name].push(value)
  } else {
    obj[name] = [obj[name], value]
  }
}

module.exports = function decodeConfig(txt) {
  var lines = txt.split(/\s*[\n\r]+\s*/g)
  var state = 'begin'
  var preNetworkComments = []
  var prop
  var network
  var conf = {
    networks: []
  }
  lines.forEach(function (line) {
    line = line.trim()
    switch (state) {
      case 'begin':
        if (line[0] === '#') {
          preNetworkComments.push(line.substr(1))
        } else if (/^network={/.test(line)) {
          state = 'network'
          network = {}
          conf.networks.push(network)
          if (preNetworkComments.length > 0) {
            network._comment = preNetworkComments.join('\n')
            preNetworkComments = []
          }
        } else if (prop = decodeProperty(line)) {
          setProp(conf, prop.name, prop.value)
          if (preNetworkComments.length > 0) {
            // should comments before global config items be handled?
            preNetworkComments = []
          }
        }
        break
      case 'network':
        if (line[0] === '#') {
          // should comments in network blocks be handled?
        } else if (line === '}') {
          state = 'begin'
        } else if (prop = decodeProperty(line)) {
          setProp(network, prop.name, prop.value)
        } else {
          throw new Error('unexpected line in network block: ' + line)
        }
        break
    }
  })
  return conf
}
