var allowMultiple = {
  domain: true,
  excluded_ssid: true,
  req_conn_capab: true,
}

var encodeValueFns = {
  anonymous_identity: encodeQuotedString,
  ca_cert: encodeQuotedString,
  client_cert: encodeQuotedString,
  domain_suffix_match: encodeQuotedString,
  imsi: encodeQuotedString,
  milenage: encodeQuotedString,
  provisioning_sp: encodeQuotedString,
  realm: encodeQuotedString,
  roaming_consortium: encodeQuotedString,
  roaming_partner: encodeQuotedString,
  username: encodeQuotedString,
  engine_id: encodeQuotedString,
  identity: encodeQuotedString,
  key_id: encodeQuotedString,
  password: encodeQuotedString,
  pcsc: encodeQuotedString,
  pin: encodeQuotedString,
  private_key: encodeQuotedString,
  private_key_passwd: encodeQuotedString,
  ssid: encodeQuotedString,
  phase1: encodeQuotedObject,
  phase2: encodeQuotedObject,
  domain: encodeQuotedString,
  excluded_ssid: encodeQuotedString,
  psk: function (value) {
    if (/[0-9a-f]{40}/i.test(value)) return value
    else return encodeQuotedString(value)
  },
}

function addProp(props, name, value) {
  if (Array.isArray(value) && name in allowMultiple) {
    value.forEach(function (val) {
      props.push({name: name, value: val})
    })
  } else {
    props.push({name: name, value: value})
  }
}

function encodeObject(obj) {
  var props = []
  for (var k in obj) {
    addProp(props, k, obj[k])
  }
  return props.map(encodeProperty).join(' ')
}

function defaultEncodeValue(value) {
  switch (typeof value) {
    case 'object':
      if (value === null)
        return ''
      if (Array.isArray(value)) {
        return value.join(' ')
      }
      return encodeObject(value)
    case 'string': return value
    case 'number': return String(value)
    case 'boolean': return value ? '1' : '0'
    case 'undefined': return ''
    default: throw new Error("weird value: " + value)
  }
}

function encodeQuotedString(value) {
  return '"' + value + '"'
}

function encodeQuotedObject(value) {
  return '"' + encodeObject(value) + '"'
}

function encodeSet(values) {
  if (!values) return ''
  if (!Array.isArray(values)) values = [values]
  return values.join(' ')
}

module.exports = function encodeConfig(conf) {
  if (!conf) return ''
  var props = []
  for (var k in conf) {
    if (k !== 'networks') {
      addProp(props, k, conf[k])
    }
  }
  var lines = props.map(encodeProperty)
  if (Array.isArray(conf.networks)) {
    lines.push('')
    conf.networks.forEach(function (network) {
      lines.push(encodeNetwork(network), '')
    })
  }
  return lines.join('\n')
}

function encodeNetwork(network) {
  if (!network) return
  var lines = []
  if (network._comment) {
    lines.push('#' + network._comment.replace(/\n/g, '\n#'))
  }
  lines.push('network={')
  var props = []
  for (var k in network) {
    if (k !== '_comment')
      addProp(props, k, network[k])
  }
  props.forEach(function (prop) {
    lines.push('\t' + encodeProperty(prop))
  })
  lines.push('}')
  return lines.join('\n')
}

function encodeProperty(prop) {
  var encodeValue = encodeValueFns[prop.name] || defaultEncodeValue
  return prop.name + '=' + encodeValue(prop.value)
}
